from django import template
from django.utils.safestring import mark_safe

register = template.Library()


@register.simple_tag(takes_context=True)
def assignee(context, ticket):
    return me_or_user(context, ticket.assignee)


@register.simple_tag(takes_context=True)
def me_or_user(context, user):
    if user == context['request'].user:
        return mark_safe('<strong>me</strong>')
    return user



from ckeditor.fields import RichTextFormField
from django import forms
from django.contrib.auth.models import User
from django.db.models import Count
from django.forms import ModelChoiceField

from helpdesk.models import State, Comment, Ticket, Project, HelpdeskProfile


class ProfileChoiceField(ModelChoiceField):
    def label_from_instance(self, obj):
        label = obj.helpdeskprofile.label if hasattr(obj, 'helpdeskprofile') else None
        return '%s (%s)' % (obj.get_full_name(), label) if label else obj.first_name

    def __init__(self, *args, **kwargs):
        queryset = User.objects.filter(groups__name='Helpdesk support').order_by('first_name')
        super(ProfileChoiceField, self).__init__(queryset, *args, **kwargs)


class CommentForm(forms.ModelForm):
    state = forms.ModelChoiceField(State.objects.all(), widget=forms.RadioSelect, initial='resolved')
    body = RichTextFormField()

    def __init__(self, *args, **kwargs):
        super(CommentForm, self).__init__(*args, **kwargs)
        self.fields['body'].widget.attrs['placeholder'] = 'Enter your answer here'
        self.fields['body'].label = 'Answer body'

    class Meta:
        model = Comment
        fields = ('body', 'state', 'internal')


class TicketForm(forms.ModelForm):
    assignee = ProfileChoiceField()

    def __init__(self, *args, **kwargs):
        super(TicketForm, self).__init__(*args, **kwargs)
        self.fields['project'].empty_label = '- None -'
        for fieldname in self.Meta.fields:
            self.fields[fieldname].show_hidden_initial = True

    class Meta:
        model = Ticket
        fields = ['assignee', 'priority', 'project', 'state']


class FilterForm(forms.Form):
    ASSIGNEES = (
        ('me', 'Me'),
        ('all', 'All')
    )
    MODES = (
        ('normal', 'Normal'),
        ('compact', 'Compact')
    )
    mode = forms.ChoiceField(choices=MODES)
    assignee = forms.ChoiceField(choices=ASSIGNEES)
    state = forms.ModelChoiceField(State.objects.all(), required=False, empty_label='All')
    project = forms.ModelChoiceField(Project.objects.all(), required=False, empty_label='All')

    def _get_user_label(self, user):
        label = user.helpdeskprofile.label if hasattr(user, 'helpdeskprofile') else None
        return '{} ({})'.format(user.first_name, label) if label else user.first_name

    def __init__(self, *args, **kwargs):
        email_filter = kwargs.pop('email_filter', False)
        view_assignees = kwargs.pop('view_assignees', False)
        super(FilterForm, self).__init__(*args, **kwargs)
        if email_filter:
            self.fields['email'] = forms.EmailField(required=False)

        if view_assignees:
            choices = User.objects.filter(ticket__isnull=False, ticket__state='open').annotate(
                tickets=Count('ticket')).order_by('-tickets')
            assignees = self.ASSIGNEES + tuple(
                (u.pk, '{} - {}'.format(self._get_user_label(u), u.tickets)) for u in choices
            )
            self.fields['assignee'].choices = assignees


class TicketCreateForm(forms.ModelForm):
    comment = RichTextFormField()
    assignee = ProfileChoiceField()

    class Meta:
        model = Ticket
        fields = ['title', 'assignee', 'priority', 'project', 'state', 'customer', 'comment']


class SearchForm(forms.Form):
    search = forms.CharField(required=False, widget=forms.TextInput(attrs={
        'placeholder': 'Search by email, title or body',
    }))



from django.conf.urls import url
from django.contrib.auth.decorators import permission_required, login_required

from helpdesk.views import HomeView, TicketView, EmailView, CommentEmailView, TicketCreateView, AttachmentView

urlpatterns = [
    url(r'^$', permission_required('helpdesk.view_tickets')(HomeView.as_view()), name='helpdesk_home'),
    url(r'^tickets/(?P<pk>\d+)/$', permission_required('helpdesk.view_tickets')(TicketView.as_view()),
        name='helpdesk_ticket'),
    url(r'^tickets/create/$', permission_required('helpdesk.add_ticket')(TicketCreateView.as_view()),
        name='helpdesk_ticket_create'),
    url(r'^tickets/(?P<pk>\d+)/email/$', permission_required('helpdesk.view_tickets')(EmailView.as_view()),
        name='helpdesk_email'),
    url(r'^comments/(?P<pk>\d+)/email/$', permission_required('helpdesk.view_tickets')(CommentEmailView.as_view()),
        name='helpdesk_comment_email'),
    url(r'^attachments/(?P<signature>[A-z0-9:]+)/', AttachmentView.as_view(), name='helpdesk_attachment')
]



# -*- encoding: utf-8 -*-
import re
from django.conf import settings

HELPDESK_DEFAULTS = {
    'username': None,
    'password': None,
    'from_email': settings.DEFAULT_FROM_EMAIL,
    'host': None,
    'mark_seen': True,
    'urlconf': None,
}

SETTINGS = HELPDESK_DEFAULTS.copy()
SETTINGS.update(getattr(settings, 'HELPDESK', {}))


class Filter(object):
    def __init__(self, request):
        super(Filter, self).__init__()
        self.request = request

    def save(self, key, value):
        self.request.session[key] = value

    def delete(self, key):
        try:
            del self.request.session[key]
        except KeyError:
            pass

    def by_assignee(self, value):
        if value not in ['me', 'all'] and not re.match(r'\d+', value):
            value = 'me'
        self.save('assignee', value)

    def by_state(self, value):
        if value is None:
            value = 'all'
        else:
            value = value.machine_name
        self.save('state', value)

    def by_project(self, value):
        if value is None:
            value = 'all'
        else:
            value = value.machine_name
        self.save('project', value)

    def by_email(self, value):
        self.save('email', value)

    def get_form_init(self):
        init = {
            'assignee': self.request.session.get('assignee', 'all'),
            'state': self.request.session.get('state', 'all'),
            'project': self.request.session.get('project', 'all'),
            'email': self.request.session.get('email', '')
        }
        return init

    def get_filters(self):
        filters = {}
        assignee = self.request.session.get('assignee', 'all')
        if not self.request.user.has_perm('helpdesk.view_all_tickets'):
            assignee = 'me'
        if assignee == 'me':
            filters['assignee'] = self.request.user
        elif assignee != 'all':
            filters['assignee__pk'] = assignee

        state = self.request.session.get('state', 'all')
        if state != 'all':
            filters['state__machine_name'] = state

        project = self.request.session.get('project', 'all')
        if project != 'all':
            filters['project__machine_name'] = project

        email = self.request.session.get('email', None)
        if email:
            filters['customer'] = email

        return filters



# -*- encoding: utf-8 -*-
from django.dispatch import Signal

new_comment_from_client = Signal(providing_args=['comment', 'ticket'])

new_answer = Signal(providing_args=['ticket', 'answer'])

ticket_pre_created = Signal(providing_args=['sent_from', 'ticket'])

ticket_post_created = Signal(providing_args=['ticket'])

ticket_updated = Signal(providing_args=['ticket', 'updater', 'changed_data', 'changes'])
